#[non_exhaustive]pub struct Command {
pub name: String,
pub args: Vec<Argument>,
pub prefix: Option<String>,
pub tags: HashSet<String>,
pub silent: bool,
pub fail: bool,
pub line_number: u32,
}Expand description
A command.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.name: StringThe name of the command. Never empty.
args: Vec<Argument>The command’s arguments, in the given order.
prefix: Option<String>The command prefix, if given.
Any command tags, if given.
silent: boolSilences the output of this command. This is handled automatically,
the Runner does not have to take this into
account.
fail: boolIf true, the command is expected to fail with a panic or error. If the command does not fail, the test fails.
line_number: u32The command’s line number position in the script.
Implementations
Sourceimpl Command
impl Command
Sourcepub fn consume_args(&self) -> ArgumentConsumer<'_> ⓘ
pub fn consume_args(&self) -> ArgumentConsumer<'_> ⓘ
Returns an argument consumer, for more convenient argument
processing. Does not affect Command::args.
See the module documentation for usage examples.