run_path

Function run_path 

pub fn run_path<R: Runner, P: AsRef<Path>>(
    runner: &mut R,
    path: P,
) -> Result<()>
Runs a testscript at the given path.

Panics if the script output differs from the current input file. Errors on IO, parser, or runner failure. If the environment variable UPDATE_TESTFILES=1 is set, the new output file will replace the input file.