Argument

Struct Argument 

#[non_exhaustive]
pub struct Argument { pub key: Option<String>, pub value: String, }
A command argument.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
key: Option<String>

The argument key, for key=value style arguments. Not guaranteed to be unique, the Runner can handle this as desired.

value: String

The argument value. Can be empty.

Implementations

Source

impl Argument

pub fn name(&self) -> &str

Returns a name for the argument – either the key, if given, or value.

pub fn parse<T>(&self) -> Result<T, Box<dyn Error>>
where T: FromStr, <T as FromStr>::Err: Display,

Parses the argument value as a T using flow::str::parse(). Convenience method that returns an improved error message as a boxed error to ease error handling in a Runner.