Command

Struct Command 

#[non_exhaustive]
pub struct Command { pub name: String, pub args: Vec<Argument>, pub prefix: Option<String>, pub tags: HashSet<String>, pub silent: bool, pub fail: bool, pub line_number: u32, }
A command.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
name: String

The name of the command. Never empty.

args: Vec<Argument>

The command’s arguments, in the given order.

prefix: Option<String>

The command prefix, if given.

tags: HashSet<String>

Any command tags, if given.

silent: bool

Silences the output of this command. This is handled automatically, the Runner does not have to take this into account.

fail: bool

If true, the command is expected to fail with a panic or error. If the command does not fail, the test fails.

line_number: u32

The command’s line number position in the script.

Implementations

Source

impl Command

pub fn consume_args(&self) -> ArgumentConsumer<'_>

Returns an argument consumer, for more convenient argument processing. Does not affect Command::args.

See the module documentation for usage examples.