Module wait

Module wait 

Constants

DEFAULT_POLL_INTERVAL
Default poll interval (1 millisecond)
DEFAULT_TIMEOSVT
Default timeout for wait operations (5 seconds)

Functions

wait_for
Wait for a condition with default timeout and poll interval
wait_for_condition
Wait for a condition to become true, polling at regular intervals