pub struct Mint { /* private fields */ }Expand description
Manages goldenfile creation and comparison for testing
Implementations
Sourceimpl Mint
impl Mint
Sourcepub fn new_with_mode<P: AsRef<Path>>(dir: P, mode: Mode) -> Self
pub fn new_with_mode<P: AsRef<Path>>(dir: P, mode: Mode) -> Self
Creates a new Mint instance for the given directory with explicit mode
Sourcepub fn new<P: AsRef<Path>>(dir: P) -> Self
pub fn new<P: AsRef<Path>>(dir: P) -> Self
Creates a new Mint instance for the given directory This method preserves backward compatibility by checking environment variables
Sourcepub fn new_goldenfile<P: AsRef<Path>>(&self, name: P) -> Result<GoldenFile>
pub fn new_goldenfile<P: AsRef<Path>>(&self, name: P) -> Result<GoldenFile>
Creates a new golden file with the given name
Sourcepub fn new_golden_file<P: AsRef<Path>>(&self, name: P) -> Result<GoldenFile>
pub fn new_golden_file<P: AsRef<Path>>(&self, name: P) -> Result<GoldenFile>
Alias for new_goldenfile for compatibility